/*
 * GpsLine.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import de.bfisch.util.Logger;
import de.bfisch.util.Propertable;


/**
 * Linie.
 */
public class GpsLine extends GpsElement implements Propertable {
    
    private static final Logger log = new Logger("GpsLine");
    
    private static Collection<GpsLine> gpsLines = new ArrayList<GpsLine>();

    private List<GpsWaypoint> gpsWaypoints;
    
    
    private GpsLine(String source) {
        super(source);
        this.gpsWaypoints = new ArrayList<GpsWaypoint>();
        log.debug("Line created.",
                new Object[] {
                   "Line", this,
        });
    }
    
    
    public static GpsLine newInstance() {
        return newInstance(null);
    }
    
    
    public static GpsLine newInstance(String source) {
        GpsLine gpsLine = new GpsLine(source);
        gpsLines.add(gpsLine);
        return gpsLine;
    }
    
    
    public static Collection instances() {
        return gpsLines;
    }
    
    
    public static void clear() {
        gpsLines.clear();
    }
    
    
    public void add(GpsWaypoint gpsWaypoint) {
        gpsWaypoints.add(gpsWaypoint);
    }
    
    
    public List getWaypoints() {
        return gpsWaypoints;
    }
    
    
    public Object[] properties() {
        int fixLength = 2 * 2;
        int entries = fixLength;
        Object[] props = new Object[entries];
        int i = 0;
        props[i++] = "size";
        props[i++] = gpsWaypoints.size();
        props[i++] = "source";
        props[i++] = getSource();
        return props;
    }

    
    public String toString() {
        StringBuffer buffer = new StringBuffer("GpsLine@");
        buffer.append(Integer.toHexString(hashCode()));
        buffer.append("[");
        buffer.append("size=");
        buffer.append(gpsWaypoints.size());
        buffer.append(",");
        buffer.append("source=");
        buffer.append(getSource());
        buffer.append("]");
        return buffer.toString();
    }
    
}
