/*
 * GpsRoute.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.bfisch.util.Logger;
import de.bfisch.util.Propertable;


public class GpsRoute extends GpsElement implements Propertable {
    
    private static final Logger log = new Logger("GpsRoute");
    
    //private static GpsIdPool routeNamePool = new GpsIdPool("Route", 12, 2);
    private static GpsIdGenerator routeNamePool = new GpsIdGenerator("Route", 12, 2);
    private static List<GpsRoute> gpsRoutes          = new ArrayList<GpsRoute>();

    private GpsLine gpsLine;
    private String name;
    private String gpsName;
    
    
    private GpsRoute(String name) {
        super(null);
        setName(name);
        log.debug("Route created.",
                new Object[] {
                   "Route", this,
        });
    }
    
    
    public static GpsRoute newInstance(String name) {
        GpsRoute gpsRoute = new GpsRoute(name);
        gpsRoutes.add(gpsRoute);
        return gpsRoute;
    }
    
    
    public static List instances() {
        return gpsRoutes;
    }

    
    public static void reset() {
        routeNamePool.reset();
        for (Iterator iterRoute = gpsRoutes.iterator(); iterRoute.hasNext(); ) {
            GpsRoute gpsRoute = (GpsRoute)iterRoute.next();
            gpsRoute.resetGpsName();
        }
    }
    
    
    public void resetGpsName() {
        gpsName = null;
    }
    
    
    public static void clear() {
        routeNamePool.clear();
        gpsRoutes.clear();
    }
    
    
    public boolean delete() {
        boolean success = gpsRoutes.remove(this);
        log.debug("Route deleted.",
                new Object[] {
                   "Route", this,
        });
        return success;
    }
    
    
    public void setName(String name) {
        this.name = name;
    }
    
    
    public String getName() {
        return name;
    }
    
    
    public String getGpsName() {
        if (gpsName == null) {
            gpsName = routeNamePool.norm(name);
        }
        return gpsName;
    }
    
    
    public void setGpsLine(GpsLine gpsLine) {
        /*
        // Ensure all waypoints have names
        List positions = gpsLine.getWaypoints();
        for (ListIterator iterPos = positions.listIterator(); iterPos.hasNext(); ) {
            GpsWaypoint gpsWaypoint = (GpsWaypoint)iterPos.next();
            if (!gpsWaypoint.isNamed()) {
                gpsWaypoint.setNamed();
            }
        }
        */
        this.gpsLine = gpsLine;
        log.debug("Route assigned line.",
                new Object[] {
                   "Route", this,
        });
    }
    
    
    public GpsLine getGpsLine() {
        return gpsLine;
    }
    
    
    public Object[] properties() {
        int fixLength = 2 * 2;
        int entries = fixLength;
        Object[] props = new Object[entries];
        int i = 0;
        props[i++] = "name";
        props[i++] = getName();
        props[i++] = "gpsLine";
        props[i++] = getGpsLine();
        return props;
    }
    
    
    public String toString() {
        StringBuffer buffer = new StringBuffer("GpsRoute@");
        buffer.append(Integer.toHexString(hashCode()));
        buffer.append("[");
        buffer.append("name=");
        buffer.append(getName());
        buffer.append(",");
        buffer.append("gpsLine=");
        buffer.append(getGpsLine());
        buffer.append("]");
        return buffer.toString();
    }
    
}
