/*
 * GpsTrack.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import de.bfisch.util.Logger;
import de.bfisch.util.Propertable;


public class GpsTrack extends GpsElement implements Propertable {
    
    private static final Logger log = new Logger("GpsTrack");
    
    //private static GpsIdPool trackNamePool = new GpsIdPool("Track", 12, 2);
    private static GpsIdGenerator trackNamePool = new GpsIdGenerator("Track", 12, 2);
    private static List<GpsTrack> gpsTracks     = new ArrayList<GpsTrack>();

    private GpsLine gpsLine;
    private String name;
    private String gpsName;
    
    
    private GpsTrack(String name, GpsLine gpsLine) {
        super(null);
        setName(name);
        setGpsLine(gpsLine);
        log.debug("Track created.",
                new Object[] {
                   "Track", this,
        });
    }
    
    
    public static GpsTrack newInstance() {
        return newInstance(null);
    }
    
    
    public static GpsTrack newInstance(String name) {
        return newInstance(name, null);
    }
    
    
    public static GpsTrack newInstance(String name, GpsLine gpsLine) {
        GpsTrack gpsTrack = new GpsTrack(name, gpsLine);
        gpsTracks.add(gpsTrack);
        return gpsTrack;
    }
    
    
    public static List instances() {
        return gpsTracks;
    }

    
    public static void reset() {
        trackNamePool.reset();
        for (Iterator iterTrack = gpsTracks.iterator(); iterTrack.hasNext(); ) {
            GpsTrack gpsTrack = (GpsTrack)iterTrack.next();
            gpsTrack.resetGpsName();
        }
    }
    
    
    public void resetGpsName() {
        gpsName = null;
    }
    
    
    public static void clear() {
        trackNamePool.clear();
        gpsTracks.clear();
    }
    
    
    public boolean delete() {
        boolean success = gpsTracks.remove(this);
        log.debug("Track deleted.",
                new Object[] {
                   "Track", this,
        });
        return success;
    }
    
    
    public void setName(String name) {
        this.name = name;
    }
    
    
    public String getName() {
        return name;
    }
    
    
    public String getGpsName() {
        if (gpsName == null) {
            gpsName = trackNamePool.norm(name);
        }
        return gpsName;
    }

    
    public void setGpsLine(GpsLine gpsLine) {
        this.gpsLine = gpsLine;
        log.debug("Track assigned line.",
                new Object[] {
                   "Track", this,
        });
    }
    
    
    public GpsLine getGpsLine() {
        return gpsLine;
    }
    
    
    public Object[] properties() {
        int fixLength = 2 * 2;
        int entries = fixLength;
        Object[] props = new Object[entries];
        int i = 0;
        props[i++] = "name";
        props[i++] = getName();
        props[i++] = "gpsLine";
        props[i++] = getGpsLine();
        return props;
    }
    
    
    public String toString() {
        StringBuffer buffer = new StringBuffer("GpsTrack@");
        buffer.append(Integer.toHexString(hashCode()));
        buffer.append("[");
        buffer.append("name=");
        buffer.append(getName());
        buffer.append(",");
        buffer.append("gpsLine=");
        buffer.append(getGpsLine());
        buffer.append("]");
        return buffer.toString();
    }
    
}
