/*
 * GpxWriter.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis;

import java.awt.geom.Point2D;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Iterator;


/**
 * Two ways to generate XML:
 *   <a href="http://java.sun.com/developer/TechTips/2000/tt0627.html#tip1">JDC Tech Tips: June 27, 2000</a>
 */
public class GpxWriter {
    
    private PrintWriter output;
    
    
    public GpxWriter(PrintWriter output) throws IOException {
        this.output = output;
        output.write("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\" ?>\n");
        output.write("<gpx xmlns=\"http://www.topografix.com/GPX/1/1\" creator=\"TopGMap\" version=\"1.0\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd\">\n");
    }
    
    
    public void write(GpsWaypoint gpsWaypoint) throws IOException {
        output.write("  \n");
        write(gpsWaypoint, "wpt", "  ", true);
    }
    
    
    private void write(GpsWaypoint gpsWaypoint, String tag, String indent, boolean withName) throws IOException {
        String name = withName ? gpsWaypoint.getGpsName() : null;
        String symbol = null;
        if (name != null) {
            if (gpsWaypoint.isBegin()) {
                symbol = "Trail Head";
            } else
            if (gpsWaypoint.isEnd()) {
                symbol = "Airport";
            } else {
                symbol = "Waypoint";
            }
        }
        Point2D pos = gpsWaypoint.getPosition();
        output.write(indent + "<" + tag + " lat=\"" + pos.getX() + "\" lon=\"" + pos.getY() + "\">\n");
        if (name != null) {
            output.write(indent + "  <name>" + name + "</name>\n");
        }
        if (symbol != null) {
            output.write(indent + "  <sym>" + symbol + "</sym>\n");
        }
        output.write(indent + "</" + tag + ">\n");
    }
    
    
    public void write(GpsRoute gpsRoute) throws IOException {
        output.write("  \n");
        output.write("  <rte>\n");
        output.write("    <name>" + gpsRoute.getGpsName() + "</name>\n");
        GpsLine gpsLine = gpsRoute.getGpsLine();
        for (Iterator iterPos = gpsLine.getWaypoints().iterator(); iterPos.hasNext(); ) {
            GpsWaypoint gpsWaypoint = (GpsWaypoint)iterPos.next();
            writeRoutePoint(gpsWaypoint, "    ");
        }
        output.write("  </rte>\n");
    }
    
    
    private void writeRoutePoint(GpsWaypoint gpsWaypoint, String indent) throws IOException {
        write(gpsWaypoint, "rtept", indent, true);
    }
    
    
    public void write(GpsTrack gpsTrack) throws IOException {
        output.write("  \n");
        output.write("  <trk>\n");
        output.write("    <name>" + gpsTrack.getGpsName() + "</name>\n");
        output.write("    <trkseg>\n");
        GpsLine gpsLine = gpsTrack.getGpsLine();
        for (Iterator iterPos = gpsLine.getWaypoints().iterator(); iterPos.hasNext(); ) {
            GpsWaypoint gpsWaypoint = (GpsWaypoint)iterPos.next();
            writeTrackPoint(gpsWaypoint, "      ");
        }
        output.write("    </trkseg>\n");
        output.write("  </trk>\n");
    }
    
    
    private void writeTrackPoint(GpsWaypoint gpsWaypoint, String indent) throws IOException {
        write(gpsWaypoint, "trkpt", indent, false);
    }
    
    
    public void close() throws IOException {
        output.write("\n");
        output.write("</gpx>\n");
        output.close();
    }
    
}