/*
 * LogEntryFormatter.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis;

import java.io.PrintWriter;
import java.io.StringWriter;
import de.bfisch.util.LogEntry;


public class LogEntryFormatter {
    
    public static final String TYPE_TIMESTAMP = LogEntry.TIMESTAMP;
    public static final String TYPE_LEVEL     = LogEntry.LEVEL;
    public static final String TYPE_DISTANCE  = "Distance";
    public static final String TYPE_EXCEPTION = "Exception";
    
    
    public LogEntryFormatter() {
    }

    
    public String objectToData(LogEntry logEntry) {
        StringBuffer plainBuf = new StringBuffer();
        
        Object[] props = logEntry.properties();
        for (int idxProps = 0; idxProps < props.length; idxProps += 2) {
            Object key       = props[idxProps];
            Object value     = props[idxProps + 1];
            String valueType = (String)key;
            String valueStr = valueToData(key, value, valueType, idxProps / 2 > 2);
            // Split lines: DOS/Windows/OS/2: "\r\n"; Apple: "\r"; UNIX: "\n"
            String[] valueLines = valueStr.split("\r\n|\r|\n");
            if (valueLines.length > 1) {
                plainBuf.deleteCharAt(plainBuf.length() - 1);
                for (int i = 0; i < valueLines.length; i++) {
                    plainBuf.append("\n\t");
                    plainBuf.append(valueLines[i]);
                }
            } else {
                plainBuf.append(valueStr);
            }
            plainBuf.append("\t");
        }
        
        return plainBuf.deleteCharAt(plainBuf.length() - 1).toString();
    }

    
    public String valueToData(Object key, Object value, String valueType, boolean keyPrefix) {
        StringBuffer buf = new StringBuffer();
        if (value != null) {
            LogEntryFormat format = LogEntryFormat.getInstance();
            String valueStr;
            if (TYPE_TIMESTAMP.equals(valueType)) {
                long timestamp = ((Long)value).longValue();
                valueStr = format.formatTimeStamp(timestamp);
            } else
            if (TYPE_LEVEL.equals(valueType)) {
                int logLevel = ((Integer)value).intValue();
                valueStr = format.formatLogLevel(logLevel);
            } else
            if (TYPE_DISTANCE.equals(valueType)) {
                double dist = ((Double)value).doubleValue();
                valueStr = format.formatDistance(dist);
            } else
            if (TYPE_EXCEPTION.equals(valueType)) {
                Throwable ex = (Throwable)value;
                StringWriter stackTrace = new StringWriter();
                ex.printStackTrace(new PrintWriter(stackTrace));
                valueStr = stackTrace.toString();
            } else {
                valueStr = value.toString();
            }
            if (keyPrefix) {
                buf.append(key);
                buf.append(": ");
            }
            buf.append(valueStr);
        }
        return buf.toString();
    }
    
}
