/*
 * JAboutDialog.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.Window;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;

import de.bfisch.gis.TopGMap;
import de.bfisch.gis.TopGMapConfig;


public class JAboutDialog extends JDialog {
    
    private static final String aboutText =
        "<html>" +
        "<p align=center><b><font size=\"+1\">TopGMap</font></b><br>" + 
        "Version " + TopGMap.VERSION + "</p>" +
        "<p align=center>&copy; Bernhard Fischer, 2005</p>" +
        "<p><a href=\"http://www.geocities.com/bfisch.geo/\">http://www.geocities.com/bfisch.geo/</a></p>" +
        "</html>";

    
    public JAboutDialog(Frame owner) {
        super(owner, "ber - TopGMap", true);
        init();
    }
    
    
    private void init() {
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        
        Container contentPane = getContentPane();
        
        JComponent textComp = createTextComponent();
        textComp.setBorder(BorderFactory.createEmptyBorder(0, 8, 4, 8));
        contentPane.add(textComp, BorderLayout.CENTER);
        
        JPanel controlPanel = createControlPanel();
        contentPane.add(controlPanel, BorderLayout.SOUTH);
                
        pack();
    }
    
    
    private JComponent createTextComponent() {
        JEditorPane editorPane = new JEditorPane();
        editorPane.setEditable(false);
        editorPane.setBackground(UIManager.getColor("Panel.background"));
        editorPane.setContentType("text/html");
        editorPane.setText(aboutText);
        editorPane.addHyperlinkListener(new HyperlinkListener() {
            public void hyperlinkUpdate(HyperlinkEvent e) {
                 if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
                     String selection = e.getURL().toExternalForm();
                     Toolkit toolkit = Toolkit.getDefaultToolkit();
                     Clipboard clipboard = toolkit.getSystemClipboard();
                     StringSelection contents = new StringSelection(selection);
                     clipboard.setContents(contents, contents);
                     JOptionPane.showMessageDialog(JAboutDialog.this, "URL in Zwischenablage kopiert.");
                 }
            }
        });
        return editorPane;
    }
    
    
    private JPanel createControlPanel() {
        JPanel controlPanel = new JPanel();
        controlPanel.setLayout(new FlowLayout());
        
        JPanel controlButtonPanel = new JPanel();
        controlButtonPanel.setLayout(new GridLayout(1, 0, 4, 0));
        controlPanel.add(controlButtonPanel);

        JButton okButton = new JButton("OK");
        controlButtonPanel.add(okButton);

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                hideDialog();
            }
        });
        
        return controlPanel;
    }
    
    
    public void showDialog() {
        if (isVisible()) {
            return;
        }
        
        Window owner = getOwner();
        setLocationRelativeTo(owner);
        setVisible(true);
    }    
    
    
    public void hideDialog() {
        dispose();
    }
    
    
    public static void main(String[] args) {
        JAboutDialog aboutDialog = new JAboutDialog(null);
        aboutDialog.setVisible(true);
    }

}
