/*
 * JLogEntryDialog.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.TransferHandler;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableColumnModelEvent;
import javax.swing.event.TableColumnModelListener;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;

import de.bfisch.gis.LogEntryFormatter;
import de.bfisch.gis.gui.JLogFrame.LogTableFormatter;
import de.bfisch.swing.BasicTableCellRenderer;
import de.bfisch.swing.BasicTableTransferHandler;
import de.bfisch.swing.GlobalCopyAction;
import de.bfisch.swing.TableFormatter;
import de.bfisch.swing.TablePopupMenuHandler;
import de.bfisch.util.Propertable;


public class JLogEntryDialog extends JDialog {


    public class LogEntryTableFormatter extends LogEntryFormatter implements TableFormatter {
      
        
        public String valueToData(JTable table, Object value, int row, int col) {
            if (row < headRows.length) {
                String valueStr = value != null ? value.toString() : "";
                return valueStr;
            } else {
                row -= headRows.length;
                Object[] props = propsObj.properties();
                String key = (String)props[2 * row];
                // Consider column reordering. 
                int modelCol = table.getColumnModel().getColumn(col).getModelIndex();
                switch (modelCol) {
                case 0:
                    return key;
                case 1:
                    String valueType = key;
                    return valueToData(key, value, valueType, false);
                }
                assert false : "Invalid column specification.";
                return null;
            }
        }
        
    }
    
    
    public class PropertyAction extends AbstractAction {
        
	
        public PropertyAction() {
            this("Properties");
        }
        
	
        public PropertyAction(String name) {
            super(name);
            logEntryTable.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                public void valueChanged(ListSelectionEvent e) {
                    int selectedRowCount = logEntryTable.getSelectedRowCount();
                    if (selectedRowCount != 1) {
                        setEnabled(false);
                        return;
                    }
                    int row = logEntryTable.getSelectedRow();
                    if (row >= headRows.length) {
                        row -= headRows.length;
                        int idxProps = 2 * row;
                        Object[] props = propsObj.properties();
                        Object key   = props[idxProps];
                        Object value = props[idxProps + 1];
                        setEnabled(value instanceof Propertable);
                    }
                }
            });
        }
        
        
        public void actionPerformed(ActionEvent e) {
            int row = logEntryTable.getSelectedRow();
            if (row >= headRows.length) {
                row -= headRows.length;
                int idxProps = 2 * row;
                Object[] props = propsObj.properties();
                Object key   = props[idxProps];
                Object value = props[idxProps + 1];
                if (value instanceof Propertable) {
                    Propertable propsObj = (Propertable)value;
                    showPropsObjDialog(key.toString(), propsObj);
                }
            }
        }
        
    }
    
    
    private static final int COL_NAME = 0;
    private static final int COL_TYPE = 1;
    
    private static String[] headRows = { "*Name", "*Typ" };    
    
    private JTable logEntryTable;
    private AbstractTableModel logEntryTableModel;
    private JScrollPane scrollPane;
    
    private String name;
    private Propertable propsObj;

    
    public JLogEntryDialog(Frame owner) {
        super(owner, "Eigenschaften - TopGMap", true);
        init();
    }

    
    public JLogEntryDialog(Dialog owner) {
        super(owner, "Eigenschaften - TopGMap", true);
        init();
    }
    
    
    private void init() {
        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        
        Container contentPane = getContentPane();
        
        logEntryTableModel = new AbstractTableModel() {
            private String[] columnNames = { "Name", "Wert" };
            
            public String getColumnName(int col) {
        	return columnNames[col];
            }
            
            public int getColumnCount() {
                return columnNames.length;
            }
            
            public int getRowCount() {
                if (propsObj == null) {
                    return 0;
                }
                Object[] props = propsObj.properties();
                int rowCount = headRows.length + props.length / 2;
                return rowCount;
            }
            
            public Object getValueAt(int row, int col) {
                if (row < headRows.length) {
                    if (col == 0) {
                        return headRows[row];
                    } else {
                        switch (row) {
                        case COL_NAME:
                            return name;
                        case COL_TYPE:
                            if (propsObj == null) return "<null>";
                            return propsObj.getClass().getName() + "@" + Integer.toHexString(propsObj.hashCode());
                        }
                        assert false;
                        return null;
                    }
                } else {
                    row -= headRows.length;
                    Object[] props = propsObj.properties();
                    int propsIndex = 2 * row + col;
                    Object value = props[propsIndex];
                    return value;
                }
            }
            
        };
        
        logEntryTable = new JTable(logEntryTableModel);
        
        TableCellRenderer logEntryTableRenderer = new BasicTableCellRenderer(new LogEntryTableFormatter());
        logEntryTable.setAutoResizeMode(JTable.AUTO_RESIZE_LAST_COLUMN);
        TableColumnModel tableColumnModel = logEntryTable.getColumnModel();
        for (int colIdx = 0; colIdx < tableColumnModel.getColumnCount(); colIdx++) {
            TableColumn tableColumn = tableColumnModel.getColumn(colIdx);
            tableColumn.setCellRenderer(logEntryTableRenderer);
            switch (colIdx) {
            case COL_NAME:
        	tableColumn.setPreferredWidth(80);
        	tableColumn.setMaxWidth(80);
        	break;
            case COL_TYPE:
        	tableColumn.setPreferredWidth(250);
        	break;
            }
        }
        
        TransferHandler logEntryTableTransferHandler =
                new BasicTableTransferHandler(BasicTableTransferHandler.FORMAT_VALUE) {
            private LogEntryTableFormatter logEntryTableFormatter = new LogEntryTableFormatter();
            
            public String valueToData(JTable table, Object value, int row, int col) {
                switch (col) {
                case COL_NAME:
                    return value.toString();
                case COL_TYPE:
                    String valueStr = logEntryTableFormatter.valueToData(table, value, row, col);
                    return valueStr;
                }
                assert false;
                return null;
            }
        };
        logEntryTable.setTransferHandler(logEntryTableTransferHandler);
        
        logEntryTable.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2) {
                    int row = logEntryTable.rowAtPoint(e.getPoint());
                    if (row >= headRows.length) {
                        row -= headRows.length;
                        int idxProps = 2 * row;
                        Object[] props = propsObj.properties();
                        Object key   = props[idxProps];
                        Object value = props[idxProps + 1];
                        if (value instanceof Propertable) {
                            Propertable propsObj = (Propertable)value;
                            showPropsObjDialog(key.toString(), propsObj);
                        }
                    }
                 }
            }
        });
        
        // Attach popup menu to logTable
        JPopupMenu logEntryTablePopupMenu = new JPopupMenu();
        logEntryTable.addMouseListener(new TablePopupMenuHandler(logEntryTablePopupMenu));
        
        JMenuItem copyMenuItem = new JMenuItem(new GlobalCopyAction("Kopieren"));
        logEntryTablePopupMenu.add(copyMenuItem);
        
        JMenuItem propertiesMenuItem = new JMenuItem(new PropertyAction("Eigenschaften"));
        logEntryTablePopupMenu.add(propertiesMenuItem);
        
        scrollPane = new JScrollPane(logEntryTable);
        contentPane.add(scrollPane, BorderLayout.CENTER);
                
        JPanel controlPanel = new JPanel();
        controlPanel.setLayout(new FlowLayout());
        contentPane.add(controlPanel, BorderLayout.SOUTH);
        
        JPanel controlButtonPanel = new JPanel();
        controlButtonPanel.setLayout(new GridLayout(1, 0, 4, 0));
        controlPanel.add(controlButtonPanel);

        JButton okButton = new JButton("Schlieen");
        controlButtonPanel.add(okButton);

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                hideDialog();
            }
        });
    }
    
    
    public void showDialog(String name, Propertable propsObj) {
        this.name = name;
        this.propsObj = propsObj;
        
        logEntryTableModel.fireTableDataChanged();
        
        if (isVisible()) {
            return;
        }
        
        Dimension size = scrollPane.getViewport().getPreferredSize();
        size.height = logEntryTable.getRowCount() * logEntryTable.getRowHeight();
        logEntryTable.setPreferredScrollableViewportSize(size);        

        //logEntryTable.setPreferredScrollableViewportSize(logEntryTable.getPreferredSize());
        
        pack();
        
        Window owner = getOwner();
        setLocationRelativeTo(owner);
        setVisible(true);
    }
    
    
    public void hideDialog() {
        dispose();
    }
    
    
    private void showPropsObjDialog(String name, Propertable propsObj) {
        JLogEntryDialog propsDialog = new JLogEntryDialog(this);
        propsDialog.showDialog(name, propsObj);
    }
    
}