/*
 * JOptionsDialog.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.gis.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import javax.swing.BorderFactory;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;

import de.bfisch.gis.TopGMapConfig;
import de.bfisch.swing.DoubleDocument;
import de.bfisch.swing.IntegerDocument;
import de.bfisch.util.Logger;


public class JOptionsDialog extends JDialog {

    private static final Logger log = new Logger("JOptionsDialog");
    private TopGMapConfig config;
    
    private JCheckBox  autoAccuracySelect;
    private JTextField posAccuracyField;
    private JTextField posAccuracyWarnField;
    
    private JCheckBox  ignoreLimitsSelect;
    private JTextField maxWaypointsField;
    private JTextField maxRoutesField;
    private JTextField maxTracksField;
    
    private JComboBox  logLevelChoice;
    
    private JButton    okButton;

    
    public JOptionsDialog(Frame owner) {
        super(owner, "Einstellungen - TopGMap", true);
        init();
    }
    
    
    private void init() {
        config = TopGMapConfig.getInstance("JOptionsDialog");

        setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
        
        Container contentPane = getContentPane();
        
        JTabbedPane tabbedPane = new JTabbedPane();
        contentPane.add(tabbedPane, BorderLayout.CENTER);
        
        JPanel accuracyPanel = createAccuracyPanel();
        accuracyPanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
        tabbedPane.addTab("Genauigkeit", accuracyPanel);
        
        JPanel devicePanel = createDevicePanel();
        devicePanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
        tabbedPane.addTab("Gert", devicePanel);
        
        JPanel logPanel = createLogPanel();
        logPanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
        tabbedPane.addTab("Protokoll", logPanel);

        JPanel controlPanel = createControlPanel();
        contentPane.add(controlPanel, BorderLayout.SOUTH);
        
        pack();
    }
    
    
    private JPanel createAccuracyPanel() {
        JPanel accuracyPanel = new JPanel();
            
        GridBagLayout gridbagAccuracy = new GridBagLayout();
        accuracyPanel.setLayout(gridbagAccuracy);
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(6, 2, 3, 2);
        
        autoAccuracySelect = new JCheckBox("Automatisch");
        autoAccuracySelect.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                boolean passed = updateState();
            }
        });
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagAccuracy.setConstraints(autoAccuracySelect, gbc);
        accuracyPanel.add(autoAccuracySelect);

        JLabel posAccuracyLabel = new JLabel("Genauigkeit");
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gridbagAccuracy.setConstraints(posAccuracyLabel, gbc);
        accuracyPanel.add(posAccuracyLabel);
        
        DoubleDocument posAccuracyDoc = new DoubleDocument();
        posAccuracyField = new JTextField(posAccuracyDoc, "", 4);
        posAccuracyField.setInputVerifier(new InputVerifier() {
            public boolean verify(JComponent input) {
                boolean passed = updateState();
                return passed;
            }
        });
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagAccuracy.setConstraints(posAccuracyField, gbc);
        accuracyPanel.add(posAccuracyField);

        JLabel posAccuracyWarnLabel = new JLabel("Annherung");
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gridbagAccuracy.setConstraints(posAccuracyWarnLabel, gbc);
        accuracyPanel.add(posAccuracyWarnLabel);
        
        DoubleDocument posAccuracyWarnDoc = new DoubleDocument();
        posAccuracyWarnField = new JTextField(posAccuracyWarnDoc, "", 4);
        posAccuracyWarnField.setInputVerifier(new InputVerifier() {
            public boolean verify(JComponent input) {
                boolean passed = updateState();
                return passed;
            }
        });
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagAccuracy.setConstraints(posAccuracyWarnField, gbc);
        accuracyPanel.add(posAccuracyWarnField);

        return accuracyPanel;
    }
    
    
    private JPanel createDevicePanel() {
        JPanel devicePanel = new JPanel();

        GridBagLayout gridbagDevice = new GridBagLayout();
        devicePanel.setLayout(gridbagDevice);
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(6, 2, 3, 2);
        
        gbc = new GridBagConstraints();
        gbc.insets = new Insets(6, 2, 3, 2);
        
        ignoreLimitsSelect = new JCheckBox("Ignorieren");
        ignoreLimitsSelect.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                boolean passed = updateState();
            }
        });
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagDevice.setConstraints(ignoreLimitsSelect, gbc);
        devicePanel.add(ignoreLimitsSelect);

        JLabel maxWaypointsLabel = new JLabel("Anzahl Wegpunkte");
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gridbagDevice.setConstraints(maxWaypointsLabel, gbc);
        devicePanel.add(maxWaypointsLabel);
        
        IntegerDocument maxWaypointsDoc = new IntegerDocument();
        maxWaypointsField = new JTextField(maxWaypointsDoc, "", 4);
        maxWaypointsField.setInputVerifier(new InputVerifier() {
            public boolean verify(JComponent input) {
                boolean passed = updateState();
                return passed;
            }
        });
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagDevice.setConstraints(maxWaypointsField, gbc);
        devicePanel.add(maxWaypointsField);
        
        JLabel maxRoutesLabel = new JLabel("Anzahl Routen");
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gridbagDevice.setConstraints(maxRoutesLabel, gbc);
        devicePanel.add(maxRoutesLabel);
        
        IntegerDocument maxRoutesDoc = new IntegerDocument();
        maxRoutesField = new JTextField(maxRoutesDoc, "", 4);
        maxRoutesField.setInputVerifier(new InputVerifier() {
            public boolean verify(JComponent input) {
                boolean passed = updateState();
                return passed;
            }
        });
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagDevice.setConstraints(maxRoutesField, gbc);
        devicePanel.add(maxRoutesField);

        JLabel maxTracksLabel = new JLabel("Anzahl Tracks");
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gridbagDevice.setConstraints(maxTracksLabel, gbc);
        devicePanel.add(maxTracksLabel);
        
        IntegerDocument maxTracksDoc = new IntegerDocument();
        maxTracksField = new JTextField(maxTracksDoc, "", 4);
        maxTracksField.setInputVerifier(new InputVerifier() {
            public boolean verify(JComponent input) {
                boolean passed = updateState();
                return passed;
            }
        });
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagDevice.setConstraints(maxTracksField, gbc);
        devicePanel.add(maxTracksField);

        return devicePanel;
    }
    
    
    private JPanel createLogPanel() {
        JPanel logPanel = new JPanel();

        GridBagLayout gridbagLog = new GridBagLayout();
        logPanel.setLayout(gridbagLog);
        
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.insets = new Insets(6, 2, 3, 2);
        
        gbc = new GridBagConstraints();
        gbc.insets = new Insets(6, 2, 3, 2);
        
        JLabel logLevelLabel = new JLabel("Protokoll-Stufe");
        gbc.anchor = GridBagConstraints.EAST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.RELATIVE;
        gridbagLog.setConstraints(logLevelLabel, gbc);
        logPanel.add(logLevelLabel);
        
        String[] logLevels = { "DEBUG", "INFO", "WARN", "ERROR", "FATAL" };
        logLevelChoice = new JComboBox(logLevels);
        gbc.anchor = GridBagConstraints.WEST;
        gbc.fill = GridBagConstraints.NONE;
        gbc.weightx = 0.0;
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gridbagLog.setConstraints(logLevelChoice, gbc);
        logPanel.add(logLevelChoice);

        return logPanel;
    }
    
    
    private JPanel createControlPanel() {
        JPanel controlPanel = new JPanel();
        controlPanel.setLayout(new FlowLayout());
        
        JPanel controlButtonPanel = new JPanel();
        controlButtonPanel.setLayout(new GridLayout(1, 0, 4, 0));
        controlPanel.add(controlButtonPanel);

        okButton = new JButton("OK");
        controlButtonPanel.add(okButton);

        okButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                hideDialog();
            }
        });
        
        JButton cancelButton = new JButton("Abbrechen");
        controlButtonPanel.add(cancelButton);

        cancelButton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dispose();
            }
        });
        
        return controlPanel;
    }
    
    
    public void showDialog() {
        if (isVisible()) {
            return;
        }

        boolean autoAccuracy = config.getAutoAccuracy();
        autoAccuracySelect.setSelected(autoAccuracy);

        double posAccuracy = config.getPosAccuracy();
        posAccuracyField.setText(Double.toString(posAccuracy));

        double posAccuracyWarn = config.getPosAccuracyWarn();
        posAccuracyWarnField.setText(Double.toString(posAccuracyWarn));

        boolean ignoreLimits = config.getIgnoreLimits();
        ignoreLimitsSelect.setSelected(ignoreLimits);
        
        int maxWaypoints = config.getMaxWaypoints();
        maxWaypointsField.setText(Integer.toString(maxWaypoints));
        
        int maxRoutes = config.getMaxRoutes();
        maxRoutesField.setText(Integer.toString(maxRoutes));
        
        int maxTracks = config.getMaxTracks();
        maxTracksField.setText(Integer.toString(maxTracks));

        int logLevel = config.getLogLevel();
        logLevelChoice.setSelectedItem(Logger.getLevelAsString(logLevel));
        
        boolean passed = updateState();
        
        Window owner = getOwner();
        setLocationRelativeTo(owner);
        setVisible(true);
    }
    
    
    public void hideDialog() {
        boolean autoAccuracy = autoAccuracySelect.isSelected();
        config.setAutoAccuracy(autoAccuracy);
        
        String posAccuracyStr = posAccuracyField.getText();
        config.setPosAccuracy(Double.parseDouble(posAccuracyStr));
        
        String posAccuracyWarnStr = posAccuracyWarnField.getText();
        config.setPosAccuracyWarn(Double.parseDouble(posAccuracyWarnStr));
        
        boolean ignoreLimits = ignoreLimitsSelect.isSelected();
        config.setIgnoreLimits(ignoreLimits);
        
        String maxWaypointsStr = maxWaypointsField.getText();
        config.setMaxWaypoints(Integer.parseInt(maxWaypointsStr));
        
        String maxRoutesStr = maxRoutesField.getText();
        config.setMaxRoutes(Integer.parseInt(maxRoutesStr));
        
        String maxTracksStr = maxTracksField.getText();
        config.setMaxTracks(Integer.parseInt(maxTracksStr));
        
        String logLevelString = (String)logLevelChoice.getSelectedItem();
        int logLevel = Logger.parseLevel(logLevelString);
        config.setLogLevel(logLevel);
        log.setLevel(logLevel);
        
        dispose();
    }
    
    
    private boolean updateState() {
        boolean passed = true;
        
        try {
            boolean autoAccuracySelected = autoAccuracySelect.isSelected();
            posAccuracyField.setEditable(!autoAccuracySelected);
            posAccuracyWarnField.setEditable(!autoAccuracySelected);
            if (!autoAccuracySelected) {
                String posAccuracyStr     = posAccuracyField.getText();
                double posAccuracy     = Double.parseDouble(posAccuracyStr);
                String posAccuracyWarnStr = posAccuracyWarnField.getText();
                double posAccuracyWarn = Double.parseDouble(posAccuracyWarnStr);
                passed &= posAccuracy <= posAccuracyWarn;
            }
            
            boolean ignoreLimitsSelected = ignoreLimitsSelect.isSelected();
            maxWaypointsField.setEditable(!ignoreLimitsSelected);
            maxRoutesField.setEditable(!ignoreLimitsSelected);
            maxTracksField.setEditable(!ignoreLimitsSelected);
            if (!ignoreLimitsSelected) {
                String maxWaypointsStr = maxWaypointsField.getText();
                int maxWaypoints = Integer.parseInt(maxWaypointsStr);
                String maxRoutesStr    = maxRoutesField.getText();
                int maxRoutes    = Integer.parseInt(maxRoutesStr);
                String maxTracksStr    = maxTracksField.getText();
                int maxTracks    = Integer.parseInt(maxTracksStr);
                passed &= maxWaypoints >= maxRoutes && maxRoutes >= maxTracks;
            }
        } catch (NumberFormatException ex) {
            passed = false;
        }
        
        okButton.setEnabled(passed);
        return passed;
    }
    
}
