/*
 * Geometry.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.math;

import java.awt.geom.Point2D;


public class Geometry {


    public static double dist(Point2D a, Point2D b, Point2D p) {
        double ax = a.getX();
        double ay = a.getY();
        double bx = b.getX();
        double by = b.getY();
        double px = p.getX();
        double py = p.getY();
        double ip = ay * bx - ax * by + ax * py - ay * px - bx * py + by * px;
        double dxab = ax - bx;
        double dyab = ay - by;
        double d = Math.sqrt((ip * ip) / (dxab * dxab + dyab * dyab));
        return d;
    }


    public static Point2D perpendicular(Point2D a, Point2D b, Point2D p) {
        double ax = a.getX();
        double ay = a.getY();
        double bx = b.getX();
        double by = b.getY();
        double px = p.getX();
        double py = p.getY();
        double dxab = bx - ax;
        double dyab = by - ay;
        if (dxab == 0) {
            double sx = ax;
            double sy = py;
            return new  Point2D.Double(sx, sy);
        }
        if (dyab == 0) {
            double sx = px;
            double sy = ay;
            return new  Point2D.Double(sx, sy);
        }
        double mab = (by - ay) / (bx - ax);
        double cab = by - mab * bx;
        double mps = - 1 / mab;
        double cps = py - mps * px;
        double sx = - (cps - cab) / (mps - mab);
        double sy = mps * sx + cps;
        return new Point2D.Double(sx, sy);
    }


    /*
    public static void main(String[] args) {
        Point2D a1 = new Point2D.Double(0.5f, 1.0f);
        Point2D b1 = new Point2D.Double(8.0f, 3.5f);
        Point2D p1 = new Point2D.Double(4.5f, 4.0f);

        Point2D a2 = new Point2D.Double(1.5f, 1.0f);
        Point2D b2 = new Point2D.Double(1.5f, 5.0f);
        Point2D p2 = new Point2D.Double(3.0f, 3.5f);

        Point2D a3 = new Point2D.Double(1.0f, 1.5f);
        Point2D b3 = new Point2D.Double(7.5f, 1.5f);
        Point2D p3 = new Point2D.Double(3.5f, 4.0f);

        Point2D a = a1;
        Point2D b = b1;
        Point2D p = p1;

        double d = dist(a, b, p);
        System.out.println("d=" + d);
        Point2D s = perpendicular(a, b, p);
        System.out.println("s=" + s);
    }
    */

}
