/*
 * GlobalCopyAction.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.swing;

import java.awt.Component;
import java.awt.Container;
import java.awt.KeyboardFocusManager;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;
import javax.swing.TransferHandler;


/**
 * @author trejkaz
 * @see http://forum.java.sun.com/thread.jspa?threadID=536113&messageID=2591499
 */
public class GlobalCopyAction extends AbstractAction {
    
    private KeyboardFocusManager focusManager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
    private Action copyAction = TransferHandler.getCopyAction();
    
    
    public GlobalCopyAction() {
        this("Copy");
    }
    
    
    public GlobalCopyAction(String name) {
        super(name);
        updateState();
        
        focusManager.addPropertyChangeListener(new PropertyChangeListener() {
            public void propertyChange(PropertyChangeEvent e) {
                String prop = e.getPropertyName();
                if (("focusOwner".equals(prop))) {
                    updateState();
                }
            }
        });
    }
    
    
    public void actionPerformed(ActionEvent e) {
	Component focusOwner = focusManager.getPermanentFocusOwner();
 
        // Simulate the action.
        copyAction.actionPerformed(
            new ActionEvent(focusOwner, e.getID(), e.getActionCommand(), e.getWhen(), e.getModifiers()));
    }
    
    
    private void updateState() {
	Component comp = focusManager.getPermanentFocusOwner();
	if (comp != null) {
            if (comp instanceof JTable) {
                JTable table = (JTable)comp;
                int selectedRowCount = table.getSelectedRowCount();
                setEnabled(selectedRowCount > 0);
            } else {
                setEnabled(false);
            }
	}
    }
    
}
