/*
 * BeanProperties.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.util;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.Enumeration;
import java.util.Properties;


public class BeanProperties extends Properties {
    
    private PropertyChangeSupport changeSupport;
    
    
    public BeanProperties(Properties defaults) {
        super(defaults);
    }
    
    
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        if (changeSupport == null) {
            changeSupport = new PropertyChangeSupport(this);
        }
        changeSupport.addPropertyChangeListener(listener);
    }
    
    
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        if (changeSupport == null) {
            changeSupport = new PropertyChangeSupport(this);
        }
        changeSupport.addPropertyChangeListener(listener);
    }
    
    
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        if (changeSupport == null) {
            changeSupport = new PropertyChangeSupport(this);
        }
        changeSupport.addPropertyChangeListener(propertyName, listener);
    }
    
    
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener) {
        if (changeSupport == null) {
            changeSupport = new PropertyChangeSupport(this);
        }
        changeSupport.addPropertyChangeListener(propertyName, listener);
    }
    
    
    public void firePropertyChange() {
        for (Enumeration propertyNameEnum = keys(); propertyNameEnum.hasMoreElements(); ) {
            String propertyName = (String)propertyNameEnum.nextElement();
            firePropertyChange(propertyName);
        }
    }
    
    
    public void firePropertyChange(String propertyName) {
        Object value = get(propertyName);
        changeSupport.firePropertyChange(propertyName, null, value);
    }
    
    
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
        if (changeSupport == null || 
            (oldValue != null && newValue != null && oldValue.equals(newValue))) {
            return;
        }
        changeSupport.firePropertyChange(propertyName, oldValue, newValue);
    }
    
    
    public void setDefaults(Properties defaults) {
        this.defaults = defaults;
    }
    
    
    public Object put(Object key, Object value) {
        Object oldValue = super.put(key, value);
        firePropertyChange((String)key, oldValue, value);
        return oldValue;
    }

}
