/*
 * Cloner.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.util;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;


public class Cloner {
    
    
    private Cloner() {
    }
    
    
    /**
     * For a discussion how to use the method clone:
     *    http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4098033
     */
    public static Object clone(Cloneable obj) throws Exception {
        if (obj instanceof Cloneable) {
            return obj.getClass().getMethod("clone", (Class[])null).invoke(obj, (Object[])null);
        } else {
            return obj;
        }
    }
    
    
    /**
     * Return a deep copy.
     */
    public static Object copy(Serializable obj) {
        try {
            ByteArrayOutputStream bOut = new ByteArrayOutputStream();
            ObjectOutputStream out = new ObjectOutputStream(bOut);
            out.writeObject(obj);
            out.close();
            
            ByteArrayInputStream bIn = new ByteArrayInputStream(bOut.toByteArray());
            ObjectInputStream in = new ObjectInputStream(bIn);
            Object newObj = in.readObject();
            in.close();
            
            return newObj;
        }
        catch (IOException ex) { ex.printStackTrace(); }
        catch (ClassNotFoundException ex) { ex.printStackTrace(); }
        assert false;
        return null;
    }
    
}
