/*
 * LogEntry.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.util;


public class LogEntry implements Propertable {
    
    public static final String TIMESTAMP = "Timestamp";
    public static final String LEVEL     = "Level";
    public static final String MESSAGE   = "Message";
    
    private long time;
    private int level;
    private String msg;
    private Object[] params;
    
    
    public LogEntry(int level, String msg, Object[] params) {
        this.time   = System.currentTimeMillis();
        this.level  = level;
        this.msg    = msg;
        this.params = params;
    }
    
    
    public long getTime() {
        return time;
    }
    
    
    public int getLevel() {
        return level;
    }
    
    
    public String getLevelAsString() {
        String levelStr = LogManager.getLevelAsString(level);
        return levelStr;
    }
    
    
    public String getMessage() {
        return msg;
    }
    
    
    public Object[] getParams() {
        return params;
    }
    
    
    public Object getValue(Object key) {
        if (params == null) {
            return null;
        }
        
        for (int i = 0; i < params.length; i += 2) {
            if (params[i].equals(key)) {
                Object value = params[i + 1];
                return value;
            }
        }
        
        return null;
    }
    
    
    public Object[] properties() {
        int fixLength = 3 * 2;
        int entries = fixLength + (params != null ? params.length : 0);
        Object[] props = new Object[entries];
        props[0] = TIMESTAMP;
        props[1] = new Long(time);
        props[2] = LEVEL;
        props[3] = new Integer(level);
        props[4] = MESSAGE;
        props[5] = msg;
        if (params != null) {
            System.arraycopy(params, 0, props, fixLength, params.length);
        }
        return props;
    }
    
}