/*
 * Logger.java
 *
 * Copyright (c) 2005 Bernhard Fischer.
 * This file is part of TopGMap. TopGMap is free software; it is
 * published under the terms of the GNU General Public License.
 */

package de.bfisch.util;

import java.io.PrintStream;


/**
 * Logging.
 */
public class Logger {
    
    private static final boolean CLONE_PARAMS = true;
    
    private static LogManager logManager = null;
    
    private String className;
    
    
    public Logger(String className) {
        this.className = className;
    }
    
    
    private LogManager getLogManager() {
        if (logManager == null) {
            logManager = LogManager.getInstance();
        }
        return logManager;
    }
    
    
    public void setLevel(int level) {
        LogManager logManager = getLogManager();
        logManager.setLevel(level);
    }
    
    
    public int getLevel() {
        LogManager logManager = getLogManager();
        int level = logManager.getLevel();
        return level;
    }
    
    
    public void setLogHandler(LogHandler logHandler) {
        LogManager logManager = getLogManager();
        logManager.setLogHandler(logHandler);
    }
    
    
    public LogHandler getLogHandler() {
        LogManager logManager = getLogManager();
        LogHandler logHandler = logManager.getLogHandler();
        return logHandler;
    }
    
    
    public void setLogStream(PrintStream logStream) {
        LogManager logManager = getLogManager();
        logManager.setLogStream(logStream);
    }
    
    
    public PrintStream getLogStream() {
        LogManager logManager = getLogManager();
        PrintStream logStream = logManager.getLogStream();
        return logStream;
    }
    
    
    public void flush() {
        LogManager logManager = getLogManager();
        logManager.flush();
    }
    
    
    public static int parseLevel(String levelStr) throws IllegalArgumentException {
        int level = LogManager.parseLevel(levelStr);
        return level;
    }
    
    
    public String getLevelAsString() {
        int level = getLevel();
        String levelStr = getLevelAsString(level);
        return levelStr;
    }
    
    
    public static String getLevelAsString(int level) {
        String levelStr = LogManager.getLevelAsString(level);
        return levelStr;
    }
    
    
    public void debug(String msg) {
        debug(msg, (Object[])null);
    }
    
    
    public void debug(String msg, Object[] params) {
        log(LogManager.DEBUG, msg, params);
    }
    
    
    public void info(String msg) {
        info(msg, null);
    }
    
    
    public void info(String msg, Object[] params) {
        log(LogManager.INFO, msg, params);
    }
    
    
    public void warn(String msg) {
        warn(msg, null);
    }
    
    
    public void warn(String msg, Object[] params) {
        log(LogManager.WARN, msg, params);
    }
    
    
    public void error(String msg) {
        error(msg, null);
    }
    
    
    public void error(String msg, Object[] params) {
        log(LogManager.ERROR, msg, params);
    }
    
    
    public void fatal(String msg) {
        fatal(msg, null);
    }
    
    
    public void fatal(String msg, Object[] params) {
        log(LogManager.FATAL, msg, params);
    }
    
    
    public void log(String msg) {
        log(msg, null);
    }
    
    
    public void log(String msg, Object[] params) {
        log(LogManager.LOG, msg, params);
    }
    
    
    public void log(int level, String msg, Object[] params) {
        if (level >= getLevel()) {
            LogManager logManager = LogManager.getInstance();
            if (CLONE_PARAMS && params != null) {
                params = (Object[])Cloner.copy(params);
            }
            LogEntry logEntry = new LogEntry(level, msg, params);
            logManager.log(logEntry);
        }
    }
    
    
    public int getDebugCount() {
        return getCount(LogManager.DEBUG);
    }
    
    
    public int getInfoCount() {
        return getCount(LogManager.INFO);
    }
    
    
    public int getWarnCount() {
        return getCount(LogManager.WARN);
    }
    
    
    public int getErrorCount() {
        return getCount(LogManager.ERROR);
    }
    
    
    public int getFatalCount() {
        return getCount(LogManager.FATAL);
    }
    
    
    public int getCount(int level) {
        LogManager logManager = LogManager.getInstance();
        int count = logManager.getCount(level);
        return count;
    }
    
    
    public String getCountSummary() {
        StringBuffer msg = new StringBuffer();
        for (int level = LogManager.FATAL; level >= LogManager.INFO; level--) {
            int count = getCount(level);
            if (count > 0) {
                if (msg.length() > 0)
                    msg.append(", ");
                msg.append(count);
                msg.append(" ");
                msg.append(LogManager.getLevelAsString(level));
                msg.append("s");
            }
        }
        return msg.toString();
    }
    
    
    public void resetCounter() {
        LogManager logManager = LogManager.getInstance();
        logManager.resetCounter();
    }
    
}